@@ -14,6 +14,7 @@ test_that("Setting/resetting options",{
   expect_equal(opt(),list(foo=3,bar=2))
   expect_equal(reset(opt), list(foo=1,bar=2))
   expect_equal(opt(),list(foo=1,bar=2))
+  expect_warning(opt(fu=1))
 })
 
 context("Local options")
@@ -37,3 +38,22 @@ test_that("is_setting",{
   expect_error(is_setting('x',foo=3))
 })
 
+context("Range checks")
+
+test_that("range checks",{
+  expect_error( options_manager(foo=1,.allowed=c(x=inrange(0,1)))  )
+  expect_error(options_manager(x=1,.allowed=list(x=inrange(2,3))))
+  opt <- options_manager(foo=1,bar=0
+    , .allowed=list(
+        foo = inrange(min=0,max=1)
+        , bar = inlist(0,1,2)
+    )
+  )
+  expect_error(opt(foo=2))
+  expect_error(opt(bar=3))
+  
+})
+
+
+
+
